-- About GetItNow.lua (version 1.60, 6/7/2012)
--
-- GetItNow.lua determines if an article is available from CCC through the Get It Now Service and displays the price and purchase options.
-- Once purchased, the information is imported into the Transaction Table.
-- Created by IDS Project (Tim Bowersox, Cyril Oberlander and Mark Sullivan) in conjunction with Copyright Clearance Center.

local settings = {};
settings.EmailAddress = GetSetting("ILLEmail");
settings.BillToEmailAddress = GetSetting("BillToEmail");
settings.OrderSource = GetSetting("OrderSource");
settings.OneClickPurchase = GetSetting("OneClickPurchase");
settings.CCCSendDirectlytoUser = GetSetting("CCCSendDirectlytoUser");
settings.Institute = GetSetting("Institute");

local interfaceMngr = nil;
local CCCDisplayForm = {};
CCCDisplayForm.Form = nil;
CCCDisplayForm.Browser = nil;
CCCDisplayForm.RibbonPage = nil;
CCCDisplayForm.GetItNowButton = nil;

local GetItNowForm = {};
GetItNowForm.Form = nil;
GetItNowForm.GetItNow = nil;

local pagesArray = {};
local ssYear = nil;
local publisher = nil;
local baseFee = nil;
local sharedServerSupport = false;
local currentSite = "ILL";

require "Atlas.AtlasHelpers";

function Init()
    if GetFieldValue("Transaction", "RequestType") == "Article" and GetFieldValue("Transaction", "ProcessType")~="Lending" then
        InitializeSharedServerSupport();
        InitializeSite();

        interfaceMngr = GetInterfaceManager();

        -- Create a form
        CCCDisplayForm.Form = interfaceMngr:CreateForm("Get It Now Search", "Script");
        GetItNowForm.Form = interfaceMngr:CreateForm("Get It Now Search", "Script");

        -- Add a browser for GetItNow and Alias
        CCCDisplayForm.Browser = CCCDisplayForm.Form:CreateBrowser("Get It Now Search", "GetItNow Browser", "Get It Now Citation");
        GetItNowForm.GetItNow = GetItNowForm.Form:CreateBrowser("GetItNow", "GetItNow Browser", "GetItNow");

        -- Hide the text label
        CCCDisplayForm.Browser.TextVisible = false;
        GetItNowForm.GetItNow.TextVisible = false;

        --Suppress Javascript errors
        CCCDisplayForm.Browser.WebBrowser.ScriptErrorsSuppressed = true;

        -- Hide CCC GetItNow browser
        GetItNowForm.GetItNow.WebBrowser.Visible=false;

        ssYear = GetFieldValue("Transaction", "PhotoJournalYear");
        if ssYear:len()>4 then
            ssYear=ssYear:match("%d%d%d%d")
        end

        -- Import Odyssey connection information if UseOdyssey set to true and version is 8.1 or higher.
        if GetSetting("UseOdyssey")==true then
            local odysseyVersion = GetCustomizationKeyValue("VersionOdysseyManager");

            if tonumber(string.sub(odysseyVersion, 0, 3))>=8.1 then
                settings.OdysseyServerIP=string.gsub(GetCustomizationKeyValue("OdysseySystemID"), "/", ":7968/");
            else
                interfaceMngr:ShowMessage("ILLiad 8.1 or higher is required to use Odyssey for Get It Now.","Upgrade Required");
                settings.OdysseyServerIP="";
            end
        else
            settings.OdysseyServerIP="";
        end
        if GetFieldValue("Transaction", "ISSN")~=nil then
            Search();
        end
    end
end

function InitializeSharedServerSupport()
    local connection = CreateManagedDatabaseConnection();

    connection.QueryString = "SELECT Value FROM Customization WHERE CustKey = 'SharedServerSupport' AND NVTGC = 'ILL'";
    connection:Connect();

    local value = connection:ExecuteScalar();

    connection:Disconnect();

    if (value == "Yes") then
        LogDebug('Shared Server Support enabled');
        sharedServerSupport = true;
    else
        LogDebug('Shared Server Support not enabled');
        sharedServerSupport = false;
    end
end

function InitializeSite()
    currentSite = "ILL";

    if sharedServerSupport then
        LogDebug('Determining shared site symbol');

        local connection = CreateManagedDatabaseConnection();
        connection.QueryString = "SELECT TABLE_SCHEMA FROM INFORMATION_SCHEMA.VIEWS WHERE TABLE_NAME = 'Users' AND TABLE_SCHEMA <> 'illiad' ORDER BY TABLE_SCHEMA";
        connection:Connect();

        currentSite = connection:ExecuteScalar();

        connection:Disconnect();
    end

    LogDebug('Using site ' .. currentSite);
end

function GetCustomizationKeyValue(key)
    if key == nil then
        return nil;
    end

    local connection = CreateManagedDatabaseConnection();
    connection.QueryString = "SELECT Value FROM Customization WHERE CustKey = '" .. key .. "'" .. " AND NVTGC = '" .. currentSite .. "'";
    connection:Connect();

    local value = connection:ExecuteScalar();

    connection:Disconnect();

    return value;
end

function Search()
    local issn = GetFieldValue("Transaction", "ISSN");
    if string.len(issn)>7 then
        GetItNowForm.GetItNow:RegisterPageHandler("custom", "testGetItNowResponse", "GetItNow", false);
        GetItNowForm.GetItNow:Navigate("https://getitnow.copyright.com/request/info?issn=" .. issn .. "&year=" .. ssYear)
    end
end

function testGetItNowResponse()
    local GetItNow_Results = GetItNowForm.GetItNow.WebBrowser.DocumentText;
    if string.find(GetItNow_Results,"Y,")~=nil then
        -- if there is a result, then we show the form.
        publisher=string.sub(GetItNow_Results, string.find(GetItNow_Results,",",3)+1)
        publisher=string.gsub (publisher, " ", "");
        if publisher==nil then
            return false;
        end
        CCCDisplayForm.Form:Show();
        return true;
    else
        return false;
    end
end

function GetItNow()
    local pages = GetFieldValue("Transaction", "PhotoJournalInclusivePages");

    pagesArray = split(pages,"-");
    if pagesArray[1]==nil then
        pagesArray[1]="boa";
    end
    if pagesArray[2]==nil then
        pagesArray[2]="eoa";
    end
    local testLoadGetItNowPage = pcall(LoadGetItNowPage);
    if  testLoadGetItNowPage==false then
        interfaceMngr:ShowMessage("Unable to process request.  Please check Publisher and Pages in the request.","Error");
    end
end

function LoadGetItNowPage()
    local CCCURL="https://getitnow.copyright.com/request?&offerIDValue=485&orderBeanReset=True&copyright=" .. publisher .. "&publicationDate=" .. ssYear .. "&author=" .. GetFieldValue("Transaction", "PhotoArticleAuthor") .. "&illEmail=" .. settings.EmailAddress .. "&contentID=" .. GetFieldValue("Transaction", "ISSN") .. " &issueNum=" .. GetFieldValue("Transaction", "PhotoJournalIssue") .. "&directURL=" .. AtlasHelpers.UrlEncode(GetFieldValue("Transaction", "PhotoArticleTitle")) .. "&startPage=" .. pagesArray[1] .. "&orderSource=" .. settings.OrderSource .. "&gin=Yes&publisherName=" .. publisher .."&institute=" .. settings.Institute .. "&endPage=" .. pagesArray[2] .. "&volumeNum=" .. GetFieldValue("Transaction", "PhotoJournalVolume") .. "&userBilled=" .. settings.BillToEmailAddress .. "&publication=" .. GetFieldValue("Transaction", "ISSN") .. "&title=" .. AtlasHelpers.UrlEncode(GetFieldValue("Transaction", "PhotoArticleTitle")) .. "&OdysseyServer=" .. settings.OdysseyServerIP  .. "&ILLiadTransactionId=" .. GetFieldValue("Transaction", "TransactionNumber");
    CCCDisplayForm.Browser:Navigate(CCCURL);
    CCCDisplayForm.Browser:RegisterPageHandler("formExists", "ecs_confirm", "PurchaseFromCCC", false);
end

function PurchaseFromCCC()
    local GetItNow_Results = GetItNowForm.GetItNow.WebBrowser.DocumentText;
    baseFee=GetItNow_Results:sub(3,string.find(GetItNow_Results,",",3)-1);
    local deliveryEmailAddress = CCCDisplayForm.Browser:GetElementInFrame(nil, "deliveryEmailAddress");

    if settings.CCCSendDirectlytoUser then
        deliveryEmailAddress:SetAttribute("value", GetFieldValue("User", "EMailAddress"));
        CCCDisplayForm.Browser:SetFormValue("ecs_confirm", "custom", GetFieldValue("User", "EMailAddress"));
    else
        deliveryEmailAddress:SetAttribute("value", settings.EmailAddress);
        CCCDisplayForm.Browser:SetFormValue("ecs_confirm", "custom", settings.EmailAddress);
    end
    local termsAgree = CCCDisplayForm.Browser:GetElementInFrame(nil, "termsAgree");
    termsAgree:SetAttribute("checked","checked");


    local tBodyElement = CCCDisplayForm.Browser.WebBrowser.Document:GetElementsByTagName("tbody");
    local myTable = CCCDisplayForm.Browser:GetElementByCollectionIndex(tBodyElement,1);
    local Elem  = CCCDisplayForm.Browser.WebBrowser.Document:CreateElement("TR");
    local TDElem  = CCCDisplayForm.Browser.WebBrowser.Document:CreateElement("TD");
    local TDElem2  = CCCDisplayForm.Browser.WebBrowser.Document:CreateElement("TD");
    TDElem.InnerHtml = "<span style='font-weight: bold;'>Get It Now Price:</span>";
    TDElem2.InnerHtml = "$" .. baseFee;
    Elem:AppendChild(TDElem);
    Elem:AppendChild(TDElem2);
    myTable:AppendChild(Elem);
    CCCDisplayForm.Browser:RegisterPageHandler("custom", "CheckIfOrdered", "ImportOrderNum", false);
end

function CheckIfOrdered()
    local bElements = CCCDisplayForm.Browser.WebBrowser.Document:GetElementsByTagName("td");
    if bElements == nil then
        return false;
    end
    for i=0, bElements.Count - 1 do
        local element = CCCDisplayForm.Browser:GetElementByCollectionIndex(bElements, i);

        if element ~= nil then
            if element.InnerText == "Order Number" then
              return true;
            end
        end --if
    end --for loop
    return false;
end

function ImportOrderNum()
    local testON = false;
    local bElements = CCCDisplayForm.Browser.WebBrowser.Document:GetElementsByTagName("td");
    local skippedSpace=false;
    if bElements == nil then
        return false;
    end

    for i=0, bElements.Count - 1 do
        local element = CCCDisplayForm.Browser:GetElementByCollectionIndex(bElements, i);

        if element ~= nil then
            if testON then
                if skippedSpace then
                    SetFieldValue("Transaction", "ILLNumber", element.InnerText);
                    testON=false;
                    skippedSpace=false;
                else
                    skippedSpace=true;
                end
            end
            if element.InnerText == "Order Number" then
              testON=true;
            end
        end --if
    end --for loop
    SetFieldValue("Transaction","CopyrightAlreadyPaid",true);
    SetFieldValue("Transaction","LendingLibrary","GETIT");
    --SetFieldValue("Transaction","LendingString","GETIT");
    SetFieldValue("Transaction","SystemID","OTH");
    SetFieldValue("Transaction","MaxCost",baseFee);
    ExecuteCommand("SwitchTab", {"Detail"});
end

function split(pString, pPattern)
    local Table = {} -- NOTE: use {n = 0} in Lua-5.0
    local fpat = "(.-)" .. pPattern
    local last_end = 1
    local s, e, cap = pString:find(fpat, 1)
    while s do
        if s ~= 1 or cap ~= "" then
            table.insert(Table,cap)
        end
        last_end = e+1
        s, e, cap = pString:find(fpat, last_end)
    end
    if last_end <= #pString then
        cap = pString:sub(last_end)
        table.insert(Table, cap)
    end
    return Table
end
